@@ -302,7 +302,7 @@ define([
             return cachedPromise;
         }
 
-        // Check the buffer to seee if this template has already been added.
+        // Check the buffer to see if this template has already been added.
         var existingBufferRecords = loadTemplateBuffer.filter(function(record) {
             return record.searchKey == searchKey;
         });
@@ -333,6 +333,50 @@ define([
         return deferred.promise();
     };
 
+    /**
+     * Prefetch a set of templates without rendering them.
+     *
+     * @param {Array} templateNames The list of templates to fetch
+     * @param {String} currentTheme
+     */
+    Renderer.prototype.prefetchTemplates = function(templateNames, currentTheme) {
+        templateNames.forEach(function(templateName) {
+            var searchKey = currentTheme + '/' + templateName;
+
+            // If we haven't already seen this template then buffer it.
+            if (getTemplatePromiseFromCache(searchKey)) {
+                return;
+            }
+
+            // Check the buffer to see if this template has already been added.
+            var existingBufferRecords = loadTemplateBuffer.filter(function(record) {
+                return record.searchKey == searchKey;
+            });
+
+            if (existingBufferRecords.length) {
+                // This template is already in the buffer so just return the existing promise.
+                // No need to add it to the buffer again.
+                return;
+            }
+
+            // This is the first time this has been requested so let's add it to the buffer to be loaded.
+            var parts = templateName.split('/');
+            var component = parts.shift();
+            var name = parts.join('/');
+
+            // Add this template to the buffer to be loaded.
+            loadTemplateBuffer.push({
+                component: component,
+                name: name,
+                theme: currentTheme,
+                searchKey: searchKey,
+                deferred: $.Deferred(),
+            });
+        });
+
+        processLoadTemplateBuffer();
+    };
+
     /**
      * Load a partial from the cache or ajax.
      *
@@ -832,6 +876,7 @@ define([
      * @param {String} newHTML - HTML to insert / replace.
      * @param {String} newJS - Javascript to run after the insertion.
      * @param {Boolean} replaceChildNodes - Replace only the childnodes, alternative is to replace the entire node.
+     * @return {Array} The list of new DOM Nodes
      */
     var domReplace = function(element, newHTML, newJS, replaceChildNodes) {
         var replaceNode = $(element);
@@ -860,7 +905,11 @@ define([
             runTemplateJS(newJS);
             // Notify all filters about the new content.
             event.notifyFilterContentUpdated(newNodes);
+
+            return newNodes.get();
         }
+
+        return [];
     };
 
     /**
@@ -999,17 +1048,23 @@ define([
      * @param {jQuery|String} element - Element or selector to prepend HTML to
      * @param {String} html - HTML to prepend
      * @param {String} js - Javascript to run after we prepend the html
+     * @return {Array} The list of new DOM Nodes
      */
     var domPrepend = function(element, html, js) {
         var node = $(element);
         if (node.length) {
             // Prepend the html.
-            node.prepend(html);
+            var newContent = $(html);
+            node.prepend(newContent);
             // Run any javascript associated with the new HTML.
             runTemplateJS(js);
             // Notify all filters about the new content.
             event.notifyFilterContentUpdated(node);
+
+            return newContent.get();
         }
+
+        return [];
     };
 
     /**
@@ -1020,17 +1075,23 @@ define([
      * @param {jQuery|String} element - Element or selector to append HTML to
      * @param {String} html - HTML to append
      * @param {String} js - Javascript to run after we append the html
+     * @return {Array} The list of new DOM Nodes
      */
     var domAppend = function(element, html, js) {
         var node = $(element);
         if (node.length) {
             // Append the html.
-            node.append(html);
+            var newContent = $(html);
+            node.append(newContent);
             // Run any javascript associated with the new HTML.
             runTemplateJS(js);
             // Notify all filters about the new content.
             event.notifyFilterContentUpdated(node);
+
+            return newContent.get();
         }
+
+        return [];
     };
 
     return /** @alias module:core/templates */ {
@@ -1053,6 +1114,25 @@ define([
             return renderer.render(templateName, context, themeName);
         },
 
+        /**
+         * Prefetch a set of templates without rendering them.
+         *
+         * @method getTemplate
+         * @param {Array} templateNames The list of templates to fetch
+         * @param {String} themeName
+         * @returns {Promise}
+         */
+        prefetchTemplates: function(templateNames, themeName) {
+            var renderer = new Renderer();
+
+            if (typeof themeName === "undefined") {
+                // System context by default.
+                themeName = config.theme;
+            }
+
+            return renderer.prefetchTemplates(templateNames, themeName);
+        },
+
         /**
          * Every call to render creates a new instance of the class and calls render on it. This
          * means each render call has it's own class variables.
@@ -1112,9 +1192,10 @@ define([
          * @param {JQuery} element - Element or selector to replace.
          * @param {String} newHTML - HTML to insert / replace.
          * @param {String} newJS - Javascript to run after the insertion.
+         * @return {Array} The list of new DOM Nodes
          */
         replaceNodeContents: function(element, newHTML, newJS) {
-            domReplace(element, newHTML, newJS, true);
+            return domReplace(element, newHTML, newJS, true);
         },
 
         /**
@@ -1124,9 +1205,10 @@ define([
          * @param {JQuery} element - Element or selector to replace.
          * @param {String} newHTML - HTML to insert / replace.
          * @param {String} newJS - Javascript to run after the insertion.
+         * @return {Array} The list of new DOM Nodes
          */
         replaceNode: function(element, newHTML, newJS) {
-            domReplace(element, newHTML, newJS, false);
+            return domReplace(element, newHTML, newJS, false);
         },
 
         /**
@@ -1136,9 +1218,10 @@ define([
          * @param {jQuery|String} element - Element or selector to prepend HTML to
          * @param {String} html - HTML to prepend
          * @param {String} js - Javascript to run after we prepend the html
+         * @return {Array} The list of new DOM Nodes
          */
         prependNodeContents: function(element, html, js) {
-            domPrepend(element, html, js);
+            return domPrepend(element, html, js);
         },
 
         /**
@@ -1148,9 +1231,10 @@ define([
          * @param {jQuery|String} element - Element or selector to append HTML to
          * @param {String} html - HTML to append
          * @param {String} js - Javascript to run after we append the html
+         * @return {Array} The list of new DOM Nodes
          */
         appendNodeContents: function(element, html, js) {
-            domAppend(element, html, js);
+            return domAppend(element, html, js);
         },
     };
 });
